



if not exist %LIBRARY_INC%\\mlir-c\\Support.h exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %LIBRARY_LIB%\\MLIRIR.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist "%LIBRARY_BIN%"\\mlir-opt.exe exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
mlir-opt -version
IF %ERRORLEVEL% NEQ 0 exit /B 1
mlir-tblgen -version
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
